/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.model.sinfo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

import org.dive4elements.river.backend.SessionHolder;
import org.dive4elements.river.model.River;
import org.hibernate.Query;
import org.hibernate.Session;

/**
 * Hibernate binding for the DB table bed_mobility
 *
 * @author Matthias Schäfer
 *
 */


@Entity
@Table(name = "bed_mobility")
public class BedMobility implements Serializable {

    /***** FIELDS *****/

    private static final long serialVersionUID = -2315936652180822551L;

    private Integer id;

    private String kmrange_info;

    private String filename;

    private String notes;

    private River river;

    private List<BedMobilityValue> values;


    /***** CONSTRUCTORS *****/

    public BedMobility() {
    }


    public BedMobility(final River river, final String filename, final String kmrange_info, final String notes) {
        this.river = river;
        this.filename = filename;
        this.kmrange_info = kmrange_info;
        this.notes = notes;
        this.values = new ArrayList<>();
    }

    /***** METHODS *****/

    @Id
    @SequenceGenerator(name = "SEQUENCE_BED_MOBILITY_ID_SEQ", sequenceName = "BED_MOBILITY_ID_SEQ", allocationSize = 1)
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQUENCE_BED_MOBILITY_ID_SEQ")
    @Column(name = "id")
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    @OneToOne
    @JoinColumn(name = "river_id")
    public River getRiver() {
        return this.river;
    }

    public void setRiver(final River river) {
        this.river = river;
    }

    @Column(name = "filename")
    public String getFilename() {
        return this.filename;
    }

    public void setFilename(final String filename) {
        this.filename = filename;
    }

    @Column(name = "kmrange_info")
    public String getKmrange_info() {
        return this.kmrange_info;
    }

    public void setKmrange_info(final String kmrange_info) {
        this.kmrange_info = kmrange_info;
    }

    @Column(name = "notes")
    public String getNotes() {
        return this.notes;
    }

    public void setNotes(final String notes) {
        this.notes = notes;
    }

    @OneToMany
    @JoinColumn(name = "bed_mobility_id")
    public List<BedMobilityValue> getValues() {
        return this.values;
    }

    public void setValues(final List<BedMobilityValue> values) {
        this.values = values;
    }

    public void addValue(final BedMobilityValue value) {
        this.values.add(value);
    }

    /**
     * Selects a bed mobility series of a river from the database
     */
    public static BedMobility queryForRiver(final River river) {
        final Session session = SessionHolder.HOLDER.get();
        final Query query = session.createQuery("FROM BedMobility WHERE river=:river");
        query.setParameter("river", river);
        final List<BedMobility> rows = query.list();
        if (rows != null)
            return rows.get(0);
        else
            return null;
    }
}